import java.util.LinkedList;

/**
 * Une classe représentant un point en 2D à coordonnées entières p=(lw, rw). <br> <br>
 * Class for representing a 2D point with integer coordinates. <br> <br>
 * Utile pour dessiner les arbres binaires (rouges et noirs)
 */
abstract class IntegerPoint2D {

  /** x and y coordinates */
  int lw, rw;

}

/**
 * A class representing a (node of a) Red-Black binary search tree
 */
class RBT extends IntegerPoint2D {
  final static boolean RED = true;
  final static boolean BLACK = false;

  boolean color;
  RBT left, right;
  String element;

  /**
   * Construit un noeud d'un arbre binaire Rouge-Noir. <br> <br>
   * Create a node of a Red-Black binary tree.
   * 
   * @param color   couleur du noeud (node color)
   * @param left    fils gauche (left child)
   * @param element élément stocké dans le noeud (element stored in the node)
   * @param right   fils droit (right child)
   */
  RBT(boolean color, RBT left, String element, RBT right) {
    this.color = color;
    this.left = left;
    this.element = element;
    this.right = right;
  }

  // Question 2.1

  /**
   * Vérifie si l'arbre en entrée satisfait propriété (2) de la définition des arbres rouges et noirs. <br> <br>
   * Check whether the input tree does satisfy the property (2) of the definitions of a red-Black tree.
   * 
   * @param t noeud racine d'un arbre rouge noir (root node a of Red-Black tree)
   * @return true  if condition (2) is satisfied for all nodes of the input tree
   */
  static boolean isRedValid(RBT t) {
    throw new Error("\nMéthode boolean isRedValid(RBT t) à compléter (Question 2.1)\nMethod boolean isRedValid(RBT t) to be completed (Question 2.1)");
  }

  // Question 2.2

  /**
   * Vérifie la validité de la condition (3) de la définition des arbres rouges et noirs. <br> <br>
   * Chech the validity of the condition (3) of the definition of Red-Black binary trees.
   * 
   * @param t noeud racine (root node)
   * @return true   si pour tout noeud 'v' de l'arbre, les chemins qui de vont de
   *         'v' jusqu'aux feuilles contiennent tous le même nombre de sommets noirs. <br> <br>
   *         (if for each node v, the paths from v to the leaves do contain the same number of black nodes)
   */
  static boolean isBlackValid(RBT t) {
    throw new Error("\nMéthode boolean isBlackValid(RBT t) à compléter (Question 2.2)\nMéthod boolean isBlackValid(RBT t) to be completed (Question 2.2)");
  }

  // Question 2.3

  /**
   * Vérifie qu'il s'agit d'un arbre rouge et noir valide (toutes les 3 conditions sont vérifiées). <br> <br>
   * Check whether the input tree is a valid Red-Black tree (all three conditions are verified).
   * 
   * @param t noeud racine (root node))
   * @return true   if the inpu tree is valid
   */
  static boolean isValid(RBT root) {
    throw new Error("\nMéthode boolean isValid(RBT t) à compléter (Question 2.3)\nMethod boolean isValid(RBT t) to be completed (Question 2.3)");
  }

  // Question 2.4

  /**
   * Estime la hauteur d'un arbre rouge et noir contenant n éléments.
   * Cette méthode renvoie l'entier non négatif h qui satisfait: 
   * <tt>
   * 2^h-1 <= n < 2^(h+1)-1
   * </tt> <br><br>
   * Estimate the height of a Red-Black tree containing n elements.
   * This method returns the non negative integer value h satisfying: 
   * <tt>
   * 2^h-1 <= n < 2^(h+1)-1
   * </tt> <br><br>
   * @param n nombre d'éléments à stocker dans l'arbre (number of elements stored
   *          in the tree)
   * @return l'hauteur de l'arbre (height )
   */
  static int estimateBlackHeight(int n) {
    assert n >= 0;
    throw new Error("\nMéthode int estimateBlackHeight(int n) à compléter (Question 2.4)\nMethod int estimateBlackHeight(int n) to completed (Question 2.4)");
  }

  // Question 2.5

  /**
   * Consruit un arbre Rouge et Noir à partir d'une liste triée. <br> <br>
   * Construct a Red-Black binary search tree starting from an input sorted list.
   * 
   * @param l  a sorted listed (elements are sorted in ascending order)
   */
  static RBT ofList(LinkedList<String> l) {
    throw new Error("\nMéthode RBT ofList(LinkedList<String> l) à compléter (Question 2.5)\nMethod RBT ofList(LinkedList<String> l) to be completed (Question 2.5)");
  }

  /**
   * Affiche les noeuds de l'arbre binaire (ordre infixe). <br> <br>
   * Print the nodes of a binary tree (prefix order).
   * 
   * @param root racine d'un arbre binaire (root of a binary tree)
   */
  public static void print(RBT root) {
    if (root != null) {
      print(root.left);
      System.out.println(root.element);
      print(root.right);
    }
  }

  /**
   * Renvoie le nombre de noeuds d'un arbre binaire rouge et noir. <br> <br>
   * Return the number of nodes of a binary tree.
   */
  static int size(RBT node) {
    if (node == null)
      return 0;
    else
      return size(node.left) + 1 + size(node.right);
  }

}
